/////////////////////////////////////////////////////////////////////////////////////////////
//
//	Callback Definitions
//
//	Used by the DLL to pass control back to the Server Application
//	Each callback must be explicitly enabled by calling one of the following
//	exported API functions:
//					EnableDisconnectNotification (DISCONNECTPROC lpCallback);
//					EnableEventMsgs (EVENTMSGPROC lpCallback);
//					SetHDAServerCallback (CWtHDACallback lpCallback); 
//		
//
//	DISCONNECTPROC
//		Notifies the Server Application whenever a client disconnects.  The DWORD argument
//		defines the number of client connections remaining.  This callback may be used
//		to shutdown the server when the last client disconnects.
//	EVENTMSGPROC
//		Allows the Application to receive event messages from the WtOPCsvr.dll for
//		tracing OPC Client Interface calls.  Primarily used for debugging purposes.
//	CWtHDACallback
//		This is the main callback function that the DLL uses whenever it needs data from
//		the application.  See the object definition below.  Override thos functions that
//		your server supports.
//
/////////////////////////////////////////////////////////////////////////////////////////////
#ifdef STRICT
typedef VOID (CALLBACK* DISCONNECTPROC)(DWORD);
typedef VOID (CALLBACK* EVENTMSGPROC)(CHAR*);
#else /* !STRICT */
typedef FARPROC DISCONNECTPROC;
typedef FARPROC EVENTMSGPROC;
#endif



#ifdef __cplusplus
extern "C" {
#endif  /* __cplusplus */

#ifdef _WTHDASVRDLL
class __declspec(dllexport) CWtHDACallback
#else
class __declspec(dllimport) CWtHDACallback
#endif
{
public:
	// This is the object definition for the main callback object
   CWtHDACallback();
   virtual ~CWtHDACallback();

// Archived Data Item functions

   // GetTimeStamps allows the DLL to retrieve a list of all the TimeStamps available
   // in the application database between the specified Start & End times.  As per the HDA
   // spec, the Start time may be greater than the End time, in which case TimeStamps must
   // be returned in reverse order.  If more data items exist in the specified time range
   // than can be filled in the allocated array, (limited by NumValues), the application
   // should return OPC_S_MOREDATA.  If the End time is not specified, (pEnd == NULL),
   // the application must use the Start time and NumValues to fill the TimeStamp array. 
   //
   // The Key value is supplied by the application to synchronize further calls to
   // GetPreviousTimeStamp, GetNextTimeStamp, and ReadAt.  When the WtHDAsvr.dll first attempts
   // to read a data set, it will pass a pointer to the session key via GetTimeStamps.  The 
   // application may then prepare the data set and return a unique identifier to the dll
   // which will be included in subsequent calls to read the data.
   virtual int GetTimeStamps (HANDLE hItem, FILETIME *pStart, FILETIME *pEnd, int NumValues, FILETIME *pTimes, DWORD *pKey);
	
   // GetPreviousTimeStamp and GetNextTimeStamp allows the DLL to retrieve the HDA Bounding
   // value for a time period as requested by a client.  The application should return the
   // Timestamp of the historized item immediately prior to or immediately following the
   // specified FILETIME.
   virtual HRESULT GetPreviousTimeStamp (HANDLE hItem, FILETIME Time, FILETIME *pTimeStamp, DWORD Key);
   virtual HRESULT GetNextTimeStamp (HANDLE hItem, FILETIME Time, FILETIME *pTimeStamp, DWORD Key);

   // ReadAt is used by the DLL to obtain the value and quality of an item at a specified time.
   // Generally, the requested timestamp will have been obtained by a previous call to
   // GetTimeStamps, however, the HDA spec allows a client to request the value of an item at
   // any given time.  In this case, the application should extrapolate the value for the 
   // item based on the surrounding data points and return OPCHDA_INTERPOLATED.  If the requested
   // timestamp is out of bounds of the database, the application should return OPC_S_NODATA.
   virtual HRESULT ReadAt (HANDLE hItem, FILETIME TimeStamp, VARIANT *pValue, DWORD *pQuality, DWORD Key);

   
// Attribute functions
   // When the application starts up, it specifies the attributes supported for 
   // the historized data items.  It also specifies, for each attribute, whether or
   // not the attribute is histroized along with the item.  The following two functions
   // allow the dll to read the current value of an attribute for a given item and, if
   // supported), the value for an attribute at a specifiied point in time.
   // 
   virtual HRESULT ReadCurrentAttribute (HANDLE hItem, DWORD AttributeID, FILETIME *pTime, VARIANT *pValue);
   virtual HRESULT ReadAttributeAt (HANDLE hItem, DWORD AttributeID, FILETIME TimeStamp, VARIANT *pValue, DWORD Key);

// Processed Item functions
   // The PrepareAggregates function allows the application to access the data and prepare
   // a list of calculated values as requested by an hda client.  The return value indicates
   // the number of calculated values, (Intervals), contained within the specified time range
   virtual int PrepareAggregates (HANDLE hItem, DWORD AggregateID, FILETIME StartTime, FILETIME EndTime, FILETIME ReSampleInterval, DWORD *pKey);

   // The ReadProcessed function allows the DLL to read a value from the application database
   // that is calculated over a specified time range.  The calculation to be performed on the
   // data set is defined by AggregateID.  The server application defines the list of
   // supported Aggregates during start-up.
   virtual HRESULT ReadProcessed (HANDLE hItem, DWORD AggregateID, FILETIME *pStart, FILETIME *pEnd, VARIANT *pValue, DWORD *pQuality, DWORD Key);

   // The QueryUpdateCapabilities function allows the DLL to determine which Update functions are
   // supported by the Server Application.  The application supplied callback object should return
   // a combination of the following:
   //					OPCHDA_INSERTCAP = 0x01,
   //					OPCHDA_REPLACECAP= 0x02,
   //					OPCHDA_INSERTREPLACECAP= 0x04,
   //					OPCHDA_DELETERAWCAP= 0x08,
   //					OPCHDA_DELETEATTIMECAP= 0x10
   virtual HRESULT QueryUpdateCapabilities(OPCHDA_UPDATECAPABILITIES *pCapabilities);

   // The Insert function is used by the DLL to write data into the HDA Server
   // as requested by the SyncUpdate::Insert or ASyncUpdate::Insert request from
   // an attached client.  The UserName is currently unavailable but reserved for
   // future use.
   virtual 	HRESULT	Insert(char *pUserName, HANDLE hItem, FILETIME TimeStamp, VARIANT *pValue, DWORD Quality); 

   // The Replace function is used by the DLL to write data into the HDA Server
   // as requested by the SyncUpdate::Replace or ASyncUpdate::Replace request from
   // an attached client.  The UserName is currently unavailable but reserved for
   // future use.
   virtual 	HRESULT	Replace(char *pUserName, HANDLE hItem, FILETIME TimeStamp, VARIANT *pValue, DWORD Quality); 

   // The InsertReplace function is used by the DLL to write data into the HDA Server
   // as requested by the SyncUpdate::InsertReplace or ASyncUpdate::InsertReplace request from
   // an attached client.  The UserName is currently unavailable but reserved for
   // future use.
   virtual 	HRESULT	InsertReplace(char *pUserName, HANDLE hItem, FILETIME TimeStamp, VARIANT *pValue, DWORD Quality); 

   // The DeleteRaw function is used by the DLL to delete data from the HDA Server
   // as requested by the SyncUpdate::IDeleteRaw or ASyncUpdate::DeleteRaw request from
   // an attached client.  The UserName is currently unavailable but reserved for
   // future use.
   virtual 	HRESULT DeleteRaw(char *pUserName, HANDLE hItem, OPCHDA_TIME *htStartTime, OPCHDA_TIME *htEndTime);

   // The DeleteAtTime function is used by the DLL to delete data from the HDA Server
   // as requested by the SyncUpdate::IDeleteAtTime or ASyncUpdate::DeleteAtTime request from
   // an attached client.  The UserName is currently unavailable but reserved for
   // future use.
   virtual 	HRESULT 	DeleteAtTime(char *pUserName, HANDLE hItem, FILETIME TimeStamp); 

};


/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////
//
//  This section represents the exported API function definitions for the WtHDAsvr.DLL//

/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//
//	Initialization and Registration functions
//
/////////////////////////////////////////////////////////////////////////////////////////////
//	WTOPCsvrRevision()
//		Simply returns a version identifier for the DLL
//
//	InitWtHDAsvr()
//		Initializes DCOM, Security, and creates the Server ClassFactory
//
//	HDAUpdateRegistry()
//		Used by the application to install itself in the Windows Registry so
//		HDA Clients can find it.
//
//	UnregisterHDAServer()
//		Un-Registers the server. Removes it from the Registry.
//
/////////////////////////////////////////////////////////////////////////////////////////////

_declspec(dllexport) BOOL WINAPI InitWtHDAsvr (GUID CLSID_Svr, UINT ServerRate);

_declspec(dllexport) BOOL WINAPI HDAUpdateRegistry (GUID CLSID_Svr, LPCSTR name, LPCSTR descr, LPCSTR exepath);

_declspec(dllexport) BOOL WINAPI UnregisterHDAServer (GUID CLSID_Svr, LPCSTR Name); 



/////////////////////////////////////////////////////////////////////////////////////////////
//
//	Callback definition functions
//
/////////////////////////////////////////////////////////////////////////////////////////////
//	SetHDAServerCallback()
//		This function establishes the main callback mechanism between the DLL and
//		the application.  When data is requested by an attached client, the DLL will
//		call one of the functions defined by the CWtHDACallback Object.
//
//	EnableHDADisconnectNotification()
//		Allows the dll to call back into the application whenever a client disconnects.
//
//	EnableHDAEventMsgs()
//		Provides a seperate callback procedure to allow the DLL to post messages back
//		to the application primarily for debugging purposes.
//
/////////////////////////////////////////////////////////////////////////////////////////////

_declspec(dllexport) BOOL WINAPI SetHDAServerCallback (CWtHDACallback *pCallback); 

_declspec(dllexport) BOOL WINAPI EnableHDADisconnectNotification (DISCONNECTPROC lpCallback);

_declspec(dllexport) BOOL WINAPI EnableHDAEventMsgs (EVENTMSGPROC lpCallback);



/////////////////////////////////////////////////////////////////////////////////////////////
//
//	Data Definition functions
//
/////////////////////////////////////////////////////////////////////////////////////////////
//	DefineHDAItem()
//		The application defines the names of the items it supports in it's historized database.
//		The DLL uses the item names to satisfy browsing requests by HDA clients.  The application
//		must maintain the returned HANDLE value to identify the item.
//
//	DefineHDAAttribute()
//		The application defines which attributes it supports for historized items.  If a
//		given attribute is saved with the item, the IsArchived parameter should be set to TRUE.
//		Otherwise, the DLL will only issue the ReadCurrentAttribute function in CWtHDACallback
//		to satisfy Attribute read requests from the client.
//
//	DefineHDAAggregate()
//		The application defines a description for each calclulation. (Aggregate), it supports.
//
//	The OPC HDA Specification defines certain ID's to be used for certain Attributes and Aggregates.
//	Support for these values is optional for any given HDA Server, however, if a server chooses to
//	define it's own set of Attributes/Aggregates, it should use ID's outside the range of the
//	pre-defined values.  Consult the OPC HDA Specification for the list of values which have
//	been pre-defined.
//
/////////////////////////////////////////////////////////////////////////////////////////////
_declspec(dllexport) HANDLE WINAPI DefineHDAItem (LPCSTR Name);

_declspec(dllexport) BOOL WINAPI DeleteHDAItem (LPCSTR Name);

_declspec(dllexport) BOOL WINAPI DefineHDAAttribute (DWORD Id, LPCSTR Name, LPCSTR Descr, VARTYPE vt, BOOL IsArchived);

_declspec(dllexport) BOOL WINAPI DefineHDAAggregate (DWORD Id, LPCSTR Name, LPCSTR Descr);




/////////////////////////////////////////////////////////////////////////////////////////////
//
//	Miscellaneous functions
//
/////////////////////////////////////////////////////////////////////////////////////////////
//	SetHDAVendorInfo()
//		Allows the application to specify a vendor specifc string to identify
//		the server.  This string will appear as part of the date returned
//		in response to the GetHistorianStatus Interface
//
//	NumbrHDAClientConnections()
//		Allows the application to determine how many clients are currently
//		connected.  May be used in the Disconnect callback procedure to shutdown
//		the server application when the last client disaonnects.
//
//	RequestHDADisconnect()
//		Allows the application to request that all clients disconnect
//
/////////////////////////////////////////////////////////////////////////////////////////////

_declspec(dllexport) void WINAPI SetHDAVendorInfo (LPCSTR VendorInfo);

_declspec(dllexport) int WINAPI NumbrHDAClientConnections ();

_declspec(dllexport) void WINAPI RequestHDADisconnect ();

/////////////////////////////////////////////////////////////////////////////////////////////
//
// SetWtHDAsvrQualifier()
//		Set delimiting character used to segment tag names
//
//
////////////////////////////////////////////////////////////////////////////////////////////

_declspec(dllexport) char  WINAPI SetWtHDAsvrQualifier (char qualifier);


/////////////////////////////////////////////////////////////////////////////////////////////
//
// Security Additions
//		Enable User Access Checks, Define the Users,
//		& Create the Items with a required access level
//
//
////////////////////////////////////////////////////////////////////////////////////////////


#define ACCESSLEVELNONE	0x00000000
#define	ACCESSLEVEL1	0x00010000
#define	ACCESSLEVEL2	0x00020000
#define	ACCESSLEVEL3	0x00040000
#define	ACCESSLEVEL4	0x00080000
#define	ACCESSLEVEL5	0x00100000
#define	ACCESSLEVEL6	0x00200000
#define	ACCESSLEVEL7	0x00400000
#define	ACCESSLEVEL8	0x00800000
#define	ACCESSLEVEL9	0x01000000
#define	ACCESSLEVEL10	0x02000000
#define	ACCESSLEVEL11	0x04000000
#define	ACCESSLEVEL12	0x08000000
#define	ACCESSLEVEL13	0x10000000
#define	ACCESSLEVEL14	0x20000000
#define	ACCESSLEVEL15	0x40000000
#define	ACCESSLEVEL16	0x80000000

_declspec(dllexport) BOOL WINAPI EnableHDASecurity (BOOL EnableSecurity, DWORD MinImpersonationLevel);

_declspec(dllexport) HRESULT WINAPI DefineHDAUser (LPCSTR GroupName, LPCSTR UserName, DWORD AccessLevel);

_declspec(dllexport) HANDLE WINAPI DefineSecureHDAItem (LPCSTR Name, DWORD ReadAccessLevel, DWORD WriteAccessLevel);



_declspec(dllexport) BOOL WINAPI DeactivateHDAsvr30MinTimer (LPCSTR Authorization);

#ifdef __cplusplus
}
#endif
